#include "FM2.h"
#include "SKINI2.msg"
namespace stk {
FM2 :: FM2( unsigned int operators )
  : nOperators_(operators)
{

  if ( nOperators_ == 0 ) 
  {
    oStream_ << "FM::FM: Number of operators must be greater than zero!";
    handleError( StkError::FUNCTION_ARGUMENT );
  }
  twozero_.setB2( -1.0 );
  twozero_.setGain( 0.0 );
  vibrato_.setFrequency( 6.0 );
  unsigned int j;
  adsr_.resize( nOperators_ );
  waves_.resize( nOperators_ );
  for (j=0; j<nOperators_; j++ ) 
  {
    ratios_.push_back( 1.0 );
    gains_.push_back( 1.0 );
    adsr_[j] = new ADSR();
  }
  modDepth_ = 0.0;
  control1_ = 1.0;
  control2_ = 1.0;
  attack0_ = 0.1;
  attack1_ = 0.1;
  attack2_ = 0.1;
  attack3_ = 0.1;
  
  decay0_ = 0.1;
  decay1_ = 0.1;
  decay2_ = 0.1;
  decay3_ = 0.1;
  
  release0_ = 0.1;
  release1_ = 0.1;
  release2_ = 0.1;
  release3_ = 0.1;
  baseFrequency_ = 440.0;
  int i;
  StkFloat temp = 1.0;
  for (i=99; i>=0; i--) 
  {
    fmGains_[i] = temp;
    temp *= 0.933033;
  }
  temp = 1.0;
  for (i=15; i>=0; i--) 
  {
    fmSusLevels_[i] = temp;
    temp *= 0.707101;
  }
  temp = 8.498186;
  for (i=0; i<32; i++) 
  {
    fmAttTimes_[i] = temp;
    temp *= 0.707101;
  }
}
FM2 :: ~FM2( void )
{
  for (unsigned int i=0; i<nOperators_; i++ ) 
  {
    delete waves_[i];
    delete adsr_[i];
  }
}
void FM2 :: loadWaves( const char **filenames )
{
  for (unsigned int i=0; i<nOperators_; i++ )
    waves_[i] = new FileLoop( filenames[i], true );
}
void FM2 :: setFrequency( StkFloat frequency )
{    
  baseFrequency_ = frequency;
  for ( unsigned int i=0; i<nOperators_; i++ )
    waves_[i]->setFrequency( baseFrequency_ * ratios_[i] );
}
void FM2 :: setRatio( unsigned int waveIndex, StkFloat ratio )
{
  if ( waveIndex >= nOperators_ ) 
  {
    oStream_ << "FM:setRatio: waveIndex parameter is greater than the number of operators!";
    handleError( StkError::WARNING ); return;
  }
  ratios_[waveIndex] = ratio;
  if (ratio > 0.0) 
    waves_[waveIndex]->setFrequency( baseFrequency_ * ratio );
  else
    waves_[waveIndex]->setFrequency( ratio );
}
void FM2 :: setGain( unsigned int waveIndex, StkFloat gain )
{
  if ( waveIndex >= nOperators_ ) 
  {
    oStream_ << "FM::setGain: waveIndex parameter is greater than the number of operators!";
    handleError( StkError::WARNING ); return;
  }
  gains_[waveIndex] = gain;
}
void FM2 :: keyOn( void )
{
  for ( unsigned int i=0; i<nOperators_; i++ )
    adsr_[i]->keyOn();
}
void FM2 :: keyOff( void )
{
  for ( unsigned int i=0; i<nOperators_; i++ )
    adsr_[i]->keyOff();
}
void FM2 :: noteOff( StkFloat amplitude )
{
  this->keyOff();
  //oStream_ << "noteOff in FM.cpp";
}
void FM2 :: controlChange( int number, StkFloat value )
{
  StkFloat normalizedValue = value * ONE_OVER_128;
  if      (number == __SK_algorithm)
    this->setAlgorithm( (unsigned int) value );
         
  else if (number == __SK_operator_0_gain)
    this->setGain0( normalizedValue );
  else if (number == __SK_operator_0_attack)
      this->setAttack0( normalizedValue );
  else if (number == __SK_operator_0_decay)
    this->setDecay0( normalizedValue );
  else if (number == __SK_operator_0_sustain)
    this->setSustain0( normalizedValue );
  else if (number == __SK_operator_0_release)
    this->setRelease0( normalizedValue );
  else if (number == __SK_operator_0_ratio)
    this->setRatio0( value );
  else if (number == __SK_operator_0_detune)
    this->setDetune0( value );

  if      (number == __SK_operator_1_gain)
    this->setGain1( normalizedValue );
  else if (number == __SK_operator_1_attack)
    this->setAttack1( normalizedValue );
  else if (number == __SK_operator_1_decay)
    this->setDecay1( normalizedValue );
  else if (number == __SK_operator_1_sustain)
    this->setSustain1( normalizedValue );
  else if (number == __SK_operator_1_release)
    this->setRelease1( normalizedValue );
  else if (number == __SK_operator_1_ratio)
    this->setRatio1( value );
  else if (number == __SK_operator_1_detune)
    this->setDetune1( value );

  if      (number == __SK_operator_2_gain)
    this->setGain2( normalizedValue );
  else if (number == __SK_operator_2_attack)
    this->setAttack2( normalizedValue );
  else if (number == __SK_operator_2_decay)
    this->setDecay2( normalizedValue );
  else if (number == __SK_operator_2_sustain)
    this->setSustain2( normalizedValue );
  else if (number == __SK_operator_2_release)
    this->setRelease2( normalizedValue );
  else if (number == __SK_operator_2_ratio)
    this->setRatio2( value );
  else if (number == __SK_operator_2_detune)
    this->setDetune2( value );

  if      (number == __SK_operator_3_gain)
    this->setGain3( normalizedValue );
  else if (number == __SK_operator_3_attack)
    this->setAttack3( normalizedValue );
  else if (number == __SK_operator_3_decay)
    this->setDecay3( normalizedValue );
  else if (number == __SK_operator_3_sustain)
    this->setSustain3( normalizedValue );
  else if (number == __SK_operator_3_release)
    this->setRelease3( normalizedValue );
  else if (number == __SK_operator_3_ratio)
    this->setRatio3( value );
  else if (number == __SK_operator_3_detune)
    this->setDetune3( value );

  else if (number == __SK_reverbLevel)
    this->setReverbLevel( value );
  else if (number == __SK_reverbTail)
    this->setReverbTail( value );
  else if (number == __SK_masterVolume)
    this->setMasterVolume( value );
}
} // stk namespace
