#ifndef STK_FM_H
#define STK_FM_H
#include "Instrmnt.h"
#include "ADSR.h"
#include "FileLoop.h"
#include "SineWave.h"
#include "TwoZero.h"


namespace stk {
class FM2 : public Instrmnt
{
 public:
  FM2( unsigned int operators = 4 );
  virtual ~FM2( void );
  void clear( void );
  void loadWaves( const char **filenames );
  virtual void setFrequency( StkFloat frequency );
  void setRatio( unsigned int waveIndex, StkFloat ratio );
  void setGain( unsigned int waveIndex, StkFloat gain );
  void setModulationSpeed( StkFloat mSpeed ) { vibrato_.setFrequency( mSpeed ); };
  void setModulationDepth( StkFloat mDepth ) { modDepth_ = mDepth; };
  void setControl1( StkFloat cVal ) { control1_ = cVal * 2.0; };
  void setControl2( StkFloat cVal ) { control2_ = cVal * 2.0; };
  void setAlgorithm(StkFloat cVal)   {algorithm_    = (unsigned int) cVal;}
  void setGain0(StkFloat cVal)   {gain0_    = cVal;}
  void setAttack0(StkFloat cVal) {attack0_  = cVal;}
  void setDecay0(StkFloat cVal)  {decay0_   = cVal;}
  void setSustain0(StkFloat cVal){sustain0_ = cVal;}
  void setRelease0(StkFloat cVal){release0_ = cVal;}
  void setRatio0(StkFloat cVal)  
  {
  if (cVal==0.0)
    ratio0_ = 0.5;
  else
    ratio0_ = cVal;
  }
  void setDetune0(StkFloat cVal) {detune0_  = 1+cVal/100.0;}
  
  void setGain1(StkFloat cVal)   {gain1_    = cVal;}
  void setAttack1(StkFloat cVal) {attack1_  = cVal;}
  void setDecay1(StkFloat cVal)  {decay1_   = cVal;}
  void setSustain1(StkFloat cVal){sustain1_ = cVal;}
  void setRelease1(StkFloat cVal){release1_ = cVal;}
  void setRatio1(StkFloat cVal)  
  {
  if (cVal==0.0)
    ratio1_ = 0.5;
  else
    ratio1_ = cVal;
  }
  void setDetune1(StkFloat cVal) {detune1_  = 1+cVal/100.0;}

  void setGain2(StkFloat cVal)   {gain2_    = cVal;}
  void setAttack2(StkFloat cVal) {attack2_  = cVal;}
  void setDecay2(StkFloat cVal)  {decay2_   = cVal;}
  void setSustain2(StkFloat cVal){sustain2_ = cVal;}
  void setRelease2(StkFloat cVal){release2_ = cVal;}
  void setRatio2(StkFloat cVal)  
  {
  if (cVal==0.0)
    ratio2_ = 0.5;
  else
    ratio2_ = cVal;
  }
  void setDetune2(StkFloat cVal) {detune2_  = 1+cVal/100.0;}

  void setGain3(StkFloat cVal)   {gain3_    = cVal;}
  void setAttack3(StkFloat cVal) {attack3_  = cVal;}
  void setDecay3(StkFloat cVal)  {decay3_   = cVal;}
  void setSustain3(StkFloat cVal){sustain3_ = cVal;}
  void setRelease3(StkFloat cVal){release3_ = cVal;}
  void setRatio3(StkFloat cVal)  
  {
  if (cVal==0.0)
    ratio3_ = 0.5;
  else
    ratio3_ = cVal;
  }
  void setDetune3(StkFloat cVal) {detune3_  = 1+cVal/100.0;}
  
  void setReverbLevel(StkFloat cVal)  {reverbLevel_   = cVal/100.0;}
  void setReverbTail(StkFloat cVal)   {reverbTail_    = cVal;}
  void setMasterVolume(StkFloat cVal) {masterVolume_  = cVal/100.0;}

  void keyOn( void );
  void keyOff( void );
  void noteOff( StkFloat amplitude );
  virtual void controlChange( int number, StkFloat value );
  virtual StkFloat tick( unsigned int ) = 0;
  virtual StkFrames& tick( StkFrames& frames, unsigned int channel = 0 ) = 0;
 protected:
  std::vector<ADSR *> adsr_; 
  std::vector<FileLoop *> waves_;
  SineWave vibrato_;
  TwoZero  twozero_;
  unsigned int nOperators_;
  StkFloat baseFrequency_;
  std::vector<StkFloat> ratios_;
  std::vector<StkFloat> gains_;
  unsigned int algorithm_;
  StkFloat modDepth_;
  StkFloat gain0_;
  StkFloat attack0_;
  StkFloat decay0_;
  StkFloat sustain0_;
  StkFloat release0_;
  StkFloat ratio0_;
  StkFloat detune0_;
  
  StkFloat gain1_;
  StkFloat attack1_;
  StkFloat decay1_;
  StkFloat sustain1_;
  StkFloat release1_;
  StkFloat ratio1_;
  StkFloat detune1_;
  
  StkFloat gain2_;
  StkFloat attack2_;
  StkFloat decay2_;
  StkFloat sustain2_;
  StkFloat release2_;
  StkFloat ratio2_;
  StkFloat detune2_;
  
  StkFloat gain3_;
  StkFloat attack3_;
  StkFloat decay3_;
  StkFloat sustain3_;
  StkFloat release3_;
  StkFloat ratio3_;
  StkFloat detune3_;
  
  StkFloat reverbLevel_;
  StkFloat reverbTail_;
  StkFloat masterVolume_;
  
  StkFloat control1_;
  StkFloat control2_;
  StkFloat fmGains_[100];
  StkFloat fmSusLevels_[16];
  StkFloat fmAttTimes_[32];
};
} // stk namespace
#endif
