/*! \class TX81Z
    \brief STK TX81Z FM synthesis instrument.
    This class implements algorithms 1 thru 8 of the TX81Z.
    -----------------------------------------------
    Algorithm Structures:
    -----------------------------------------------
    Carrier = first factor of each first level term
    Modulation is by right-multiplication
    FeedBack In  = ]
    FeedBack Out = [
    -----------------------------------------------
    1 - 012[3]
    2 - 01(2+[3])
    3 - 0(12+[3])
    4 - 0(1+2[3])
    5 - 01+2[3]
    6 - 0[3]+1[3]+2[3]
    7 - 0+1+2[3]
    8 - 0+1+2+[3]
    -----------------------------------------------
    Control Change Numbers: 
       - Total Modulator Index = 2
       - Modulator Crossfade = 4
       - LFO Speed = 11
       - LFO Depth = 1
       - ADSR 2 & 4 Target = 128
*/
#include "TX81Z.h"
namespace stk {
TX81Z :: TX81Z( void )
  : FM2()
{
  for ( unsigned int i=0; i<4; i++ )
    waves_[i] = new FileLoop( (Stk::rawwavePath() + "sinewave.raw").c_str(), true );
  this->setRatio(0, 1.00);
  this->setRatio(1, 2.00);
  this->setRatio(2, 4.00);
  this->setRatio(3, 8.00);
  gains_[0] = fmGains_[99];
  gains_[1] = fmGains_[99];
  gains_[2] = fmGains_[99];
  gains_[3] = fmGains_[99];
  adsr_[0]->setAllTimes(0.01, 0.1, 0.5, 0.2);
  adsr_[1]->setAllTimes(0.01, 0.1, 0.5, 0.2);
  adsr_[2]->setAllTimes(0.01, 0.1, 0.5, 0.2);
  adsr_[3]->setAllTimes(0.01, 0.1, 0.5, 0.2);
  twozero_.setGain( 0.5 );
  modDepth_ = 0.040;
  
}  
TX81Z :: ~TX81Z( void )
{
 
}
void TX81Z :: setFrequency( StkFloat frequency )
{   

  baseFrequency_ = frequency;
}
void TX81Z :: noteOn( StkFloat frequency, StkFloat amplitude )
{
  gains_[0] = gain0_;
  gains_[1] = gain1_;
  gains_[2] = gain2_;
  gains_[3] = gain3_;
  this->setRatio(0, ratio0_ * detune0_);
  this->setRatio(1, ratio1_ * detune1_);
  this->setRatio(2, ratio2_ * detune2_);
  this->setRatio(3, ratio3_ * detune3_);
  adsr_[0]->setAllTimes(attack0_, decay0_, sustain0_, release0_);
  adsr_[1]->setAllTimes(attack1_, decay1_, sustain1_, release1_);
  adsr_[2]->setAllTimes(attack2_, decay2_, sustain2_, release2_);
  adsr_[3]->setAllTimes(attack3_, decay3_, sustain3_, release3_);
  this->setFrequency( frequency );
  this->keyOn();
}
} // stk namespace
